import string
import re
import os

import nltk
nltk.download('stopwords', quiet=True)
nltk.download('punkt', quiet=True)
nltk.download('wordnet', quiet=True)
nltk.download('omw-1.4', quiet=True)

import pandas as pd

from nltk import word_tokenize
from nltk.corpus import stopwords
from unidecode import unidecode
from nltk.stem import WordNetLemmatizer
from nltk.tokenize import word_tokenize

LANGUAGES_STOPWORDS = ['english', 'spanish', 'portuguese']
EXCEPTION_PUNCTUATION = ['-']
WORDS_TO_KEEP = ['only', 'between', 'have', 'having', 'all', 'any']

AMENITIES = []
if os.path.isfile('data/amenities.parquet'):
    df_amenities = pd.read_parquet('data/amenities.parquet')
    AMENITIES = list(df_amenities['amenity'])        

def pre_process(text, lemma=True, log=False):
    
    print('ORIGINAL:', text) if log else None
    
    # Convert to lower case
    text = text.lower()
    print('LOWER:', text) if log else None
    
    # Remove non-ascii characters
    text = unidecode(text)
    # Remove hex values
    text = re.sub(r'[^\w]', ' ', text).strip()
    print('NON-ASCII:', text) if log else None
    
    # Remove punctuation
    for punct in list(string.punctuation):
        new_char = '' if punct in EXCEPTION_PUNCTUATION else ' '
        text = text.replace(punct, new_char)
    print('PUNCT:', text) if log else None
    
    # Remove isolated ignored punctuations
    for exc in EXCEPTION_PUNCTUATION:
        text = text.replace(f' {exc} ', '')
    print('ISOLATED:', text) if log else None
    
    # Separate numbers and letters
    text = re.sub(r"([0-9]+(\.[0-9]+)?)",r" \1 ", text).strip()
    
    # Remove stopwords from text
    stopset = []
    for language in LANGUAGES_STOPWORDS:
        stopset += stopwords.words(language)
    # Keep some specific words
    for word in WORDS_TO_KEEP:
        stopset.remove(word)
        
    phrase = []
    for word in word_tokenize(text):
        # Remove leadings zeros
        word = word.lstrip('0')
        if word not in stopset and word not in AMENITIES:
            phrase.append(word)
    text = " ".join(phrase)
    print('STOPWORDS:', text) if log else None
    
    # Join NON terms
    text = text.replace('non ', 'non')
    print('NON:', text) if log else None
    
    if lemma:
        text = lemmatize(text)
        print('LEMMA:', text) if log else None
    
    return text


def lemmatize(text):
    
    lemmatizer = WordNetLemmatizer()

    words = word_tokenize(text)
    for w in words:
        w = lemmatizer.lemmatize(w)

    new_sentence = " ".join(words)

    return new_sentence
